/* PDF image extractor based on PDFlib TET
 *
 * $Id: image_extractor.c,v 1.1 2013/05/13 11:40:23 tm Exp $
 */

#include <stdio.h>

#include "tetlib.h"

/* global option list */
static const char *globaloptlist = "searchpath={{../data}}";

/* document-specific option list */
static const char *docoptlist = "";

/* page-specific option list;
 * Since we extract only images we disable the text engine
 */
static const char *pageoptlist = "skipengines={text}";

/* here you can insert basic image extract options (more below) */
static const char *baseimageoptlist = "";

int main(int argc, char **argv)
{
    TET *tet;
    volatile int pageno = 0;
    char *outfilebase;

    if (argc != 2)
    {
        fprintf(stderr, "usage: image_extractor <infilename>\n");
        return(2);
    }

    outfilebase = argv[1];

    if ((tet = TET_new()) == (TET *) 0)
    {
        fprintf(stderr, "image_extractor: out of memory\n");
        return(2);
    }

    
    TET_TRY (tet)
    {
        int n_pages;
        int doc;

        TET_set_option(tet, globaloptlist);

        doc = TET_open_document(tet, argv[1], 0, docoptlist);

        if (doc == -1)
        {
            fprintf(stderr, "Error %d in %s(): %s\n",
                TET_get_errnum(tet), TET_get_apiname(tet), TET_get_errmsg(tet));
            TET_EXIT_TRY(tet);
            TET_delete(tet);
            return(2);
        }

        /* Get number of pages in the document */
        n_pages = (int) TET_pcos_get_number(tet, doc, "length:pages");

	/* loop over pages in the document */
        for (pageno = 1; pageno <= n_pages; ++pageno)
        {
	    const TET_image_info *ti;
            int page;

            page = TET_open_page(tet, doc, pageno, pageoptlist);

            if (page == -1)
            {
                fprintf(stderr, "Error %d in %s() on page %d: %s\n",
                    TET_get_errnum(tet), TET_get_apiname(tet), pageno,
                    TET_get_errmsg(tet));
                continue;                        /* process next page */
            }

	    /* Retrieve all images on the page */
            while ((ti = TET_get_image_info(tet, page)) != 0)
            {
		char imageoptlist[1024];
		int cs;

		/* Report image geometry */
		printf("page %d: %dx%dpt, alpha=%g, beta=%g\n",
		    pageno, (int) ti->width, (int) ti->height,
		    ti->alpha, ti->beta);

		/* Retrieve additional image properties with pCOS */
		printf("   id=%d, %dx%d pixel, ",
		    ti->imageid,
		    (int) TET_pcos_get_number(tet, doc,
				    "images[%d]/Width", ti->imageid),
		    (int) TET_pcos_get_number(tet, doc,
				    "images[%d]/Height", ti->imageid));

		cs = (int) TET_pcos_get_number(tet, doc,
				    "images[%d]/colorspaceid", ti->imageid);

		if (cs != -1)
		{
		    printf("%dx%d bit %s\n",
			    (int) TET_pcos_get_number(tet, doc,
				    "colorspaces[%d]/components", cs),
			    (int) TET_pcos_get_number(tet, doc,
				    "images[%d]/bpc", ti->imageid),
			    TET_pcos_get_string(tet, doc,
				    "colorspaces[%d]/name", cs));
		}
		else
		{
		    /* cs==-1 may happen for some JPEG 2000 images. bpc,
		     * colorspace name and number of components are not
		     * available in this case.
		     */
		    printf("JPEG2000\n");
		}

		/*
		 * Fetch image data and write it to a disk file. The
		 * output filename is generated from the input filename,
		 * page number and image ID.
		 */
		sprintf(imageoptlist, "%s filename={%s_p%d_I%d}",
		    baseimageoptlist, outfilebase, pageno, ti->imageid);

		if (TET_write_image_file(tet, doc, ti->imageid, imageoptlist)
		    == -1)
		{
		    printf("\nError %d in %s(): %s\n",
			TET_get_errnum(tet), TET_get_apiname(tet),
			TET_get_errmsg(tet));
		    continue;                  /* process next image */
		}
            }

            if (TET_get_errnum(tet) != 0)
            {
                fprintf(stderr, "Error %d in %s() on page %d: %s\n",
                    TET_get_errnum(tet), TET_get_apiname(tet), pageno,
                    TET_get_errmsg(tet));
            }

            TET_close_page(tet, page);
        }

        TET_close_document(tet, doc);
    }

    TET_CATCH (tet)
    {
        if (pageno == 0)
            fprintf(stderr, "Error %d in %s(): %s\n",
                TET_get_errnum(tet), TET_get_apiname(tet), TET_get_errmsg(tet));
        else
            fprintf(stderr, "Error %d in %s() on page %d: %s\n",
                TET_get_errnum(tet), TET_get_apiname(tet), pageno,
                TET_get_errmsg(tet));
    }

    TET_delete(tet);

    return 0;
}
